@@ -89,6 +89,10 @@ mod222logisticcmw_2_1 <- STVAR(data=gdpdef, p=2, M=2, d=2, params=theta_222logis
                                weightfun_pars=c(2, 1), mean_constraints=list(1:2), AR_constraints=C_222, parametrization="mean",
                                weight_constraints=list(R=matrix(c(0, 1), nrow=2), r=c(0.01, 0)))
 
+tmppars <- c(theta_222relg[-length(theta_222relg)], c(0.75, 100))
+tmpmod <- STVAR(data=gdpdef, p=2, M=2, d=2, params=tmppars, weight_function="exponential", weightfun_pars=c(2, 1))
+plot(tmpmod)
+
 # p=2, M=2, d=2, weight_function="exponential", weightfun_pars=c(2, 1), mean_constraints=list(1:2), AR_constraints=C_222,
 # weight_constraints=list(R=matrix(c(0, 1), nrow=2), r=c(0.01, 0))
 xi_222expcmw_2_1 <- c(0.33)
@@ -146,6 +150,22 @@ mod222logistit <- STVAR(data=gdpdef, p=2, M=2, d=2, params=theta_222logistit, we
                         cond_dist="ind_Student", mean_constraints=list(1:2), AR_constraints=C_222,
                         weight_constraints=list(R=matrix(c(0, 1), nrow=2), r=c(0.01, 0)), parametrization="mean")
 
+## ind_skewed_t
+
+# p=1, M=2, d=3, weight_function="exogenous", weighfun_pars=weightfun_pars123, cond_dist="ind_skewed_t"
+theta_123exoikt <- c(0.10741, 0.13813, -0.12092, 3.48957, 0.60615, 0.45646, 0.87227, -0.01595, 0.14124,
+                    -0.08611, 0.61865, 0.34311, -0.02047, 0.025, 0.97548, 0.74976, 0.02187, 0.29213,
+                    -1.55165, 0.58245, -0.00696, -0.07261, 0.02021, 0.96883, Bmatpars123, 7, 3, 13, -0.1, 0.2, 0.3)
+mod123exoikt <- STVAR(data=usamone, p=1, M=2, d=3, params=theta_123exoikt, weight_function="exogenous",
+                      weightfun_pars=weightfun_pars123, cond_dist="ind_skewed_t")
+
+# p=2, M=2, d=2, weight_function="logistic", weightfun_pars=c(2, 1), cond_dist="ind_skewed_t", mean_constraints=list(1:2),
+# AR_constraints=C_222, weight_constraints=list(R=matrix(c(0, 1), nrow=2), r=c(0.01, 0)), parametrization="mean"
+theta_222logistikt <- c(0.7209658, 0.810858, 0.22, 0.06, -0.15, 0.39, 0.41, -0.01, 0.08, 0.3, Bmatpars222, 0.4, 7, 3, 0.1, 0)
+mod222logistikt <- STVAR(data=gdpdef, p=2, M=2, d=2, params=theta_222logistikt, weight_function="logistic", weightfun_pars=c(2, 1),
+                        cond_dist="ind_skewed_t", mean_constraints=list(1:2), AR_constraints=C_222,
+                        weight_constraints=list(R=matrix(c(0, 1), nrow=2), r=c(0.01, 0)), parametrization="mean")
+
 
 ## Structural models
 
@@ -212,6 +232,17 @@ mod222logistitb <- STVAR(data=gdpdef, p=2, M=2, d=2, params=theta_222logistitb,
                         weight_constraints=list(R=matrix(c(0, 1), nrow=2), r=c(0.01, 0)), identification="non-Gaussianity",
                         parametrization="mean", B_constraints=matrix(c(1, NA, 0, 1), nrow=2, ncol=2))
 
+# p=2, M=2, d=2, weight_function="logistic", weightfun_pars=c(2, 1), cond_dist="ind_skewed_t", mean_constraints=list(1:2),
+# AR_constraints=C_222, weight_constraints=list(R=matrix(c(0, 1), nrow=2), r=c(0.01, 0)), parametrization="mean",
+# B_constraints=matrix(c(1, NA, 0, 1), nrow=2, ncol=2)
+theta_222logistiktb <- c(0.7209658, 0.810858, 0.22, 0.06, -0.15, 0.39, 0.41, -0.01, 0.08, 0.3, # mu + A
+                        0.1, 0.2, 0.7, 0.11, -0.22, 0.73, # B mats
+                        0.4, 7, 3, 0.4, 0.7)
+mod222logistiktb <- STVAR(data=gdpdef, p=2, M=2, d=2, params=theta_222logistiktb, weight_function="logistic", weightfun_pars=c(2, 1),
+                         cond_dist="ind_skewed_t", mean_constraints=list(1:2), AR_constraints=C_222,
+                         weight_constraints=list(R=matrix(c(0, 1), nrow=2), r=c(0.01, 0)), identification="non-Gaussianity",
+                         parametrization="mean", B_constraints=matrix(c(1, NA, 0, 1), nrow=2, ncol=2))
+
 
 test_that("STVAR works correctly", {
   # Relative_dens Gaussian STVAR
@@ -248,6 +279,10 @@ test_that("STVAR works correctly", {
   expect_equal(mod123exoit$params, theta_123exoit)
   expect_equal(mod222logistit$params, theta_222logistit)
 
+  # ind_skewed_t
+  expect_equal(mod123exoikt$params, theta_123exoikt)
+  expect_equal(mod222logistikt$params, theta_222logistikt)
+
   # Structural
   expect_equal(mod222thressr_2_1$params, theta_222thres_2_1)
   expect_equal(mod122relgsh$params, theta_122relgsh)
@@ -255,6 +290,7 @@ test_that("STVAR works correctly", {
   expect_equal(mod222expcmwtsh_2_1$params, theta_222expcmwtsh_2_1)
   expect_equal(mod222expcmwbtsh_2_1$params, theta_222expcmwbtsh_2_1)
   expect_equal(mod222logistitb$params, theta_222logistitb)
+  expect_equal(mod222logistiktb$params, theta_222logistiktb)
 })
 
 test_that("swap_parametrization works correctly", {
@@ -293,8 +329,16 @@ test_that("swap_parametrization works correctly", {
                  0.59600000, 1.63600000, 0.68900000, -1.28100000, -0.21300000, 1.89700000, 1.77700000, 0.56700000, 0.01600000,
                  0.38300000, -0.04500000, 0.03400000, 0.16900000, 1.16500000, -0.04400000, 7.00000000, 3.00000000, 13.00000000),
                tolerance=1e-3)
-})
 
+  # ind_skewed_t exo
+  expect_equal(swap_parametrization(mod123exoikt, calc_std_errors=FALSE)$params,
+               c(0.01872735, 0.54692881, 2.82960017, -0.50345541, 1.88535674, 9.50476379, 0.87227000, -0.01595000, 0.14124000,
+                 -0.08611000, 0.61865000, 0.34311000, -0.02047000, 0.02500000, 0.97548000, 0.74976000, 0.02187000, 0.29213000,
+                 -1.55165000, 0.58245000, -0.00696000, -0.07261000, 0.02021000, 0.96883000, 0.21700000, -0.54200000, 0.89100000,
+                 0.59600000, 1.63600000, 0.68900000, -1.28100000, -0.21300000, 1.89700000, 1.77700000, 0.56700000, 0.01600000,
+                 0.38300000, -0.04500000, 0.03400000, 0.16900000, 1.16500000, -0.04400000, 7.00000000, 3.00000000, 13.00000000,
+                 -0.1, 0.2, 0.3), tolerance=1e-3)
+})
 
 
 
@@ -352,6 +396,23 @@ test_that("reorder_B_columns works correctly", {
 
   expect_equal(reorder_B_columns(mod222logistitb, perm=c(2, 1), calc_std_errors=FALSE)$model$B_constraints,
                matrix(c(0, 1, 1, NA), nrow=2), tolerance=1e-4)
+
+  # ind_skewed_t
+  expect_equal(reorder_B_columns(mod123exoikt, perm=c(3, 1, 2), calc_std_errors=FALSE)$params,
+               c(0.10741, 0.13813, -0.12092, 3.48957, 0.60615, 0.45646, 0.87227, -0.01595, 0.14124, -0.08611, 0.61865, 0.34311,
+                 -0.02047, 0.025, 0.97548, 0.74976, 0.02187, 0.29213, -1.55165, 0.58245, -0.00696, -0.07261, 0.02021, 0.96883,
+                 -1.281, -0.213, 1.897, 0.217, -0.542, 0.891, 0.596, 1.636, 0.689, 0.169, 1.165, -0.044, 1.777, 0.567, 0.016,
+                 0.383, -0.045, 0.034, 13, 7, 3, 0.3, -0.1, 0.2), tolerance=1e-4)
+
+  expect_equal(reorder_B_columns(mod222logistiktb, perm=c(2, 1), calc_std_errors=FALSE)$params,
+               c(0.7209658, 0.810858, 0.22, 0.06, -0.15, 0.39, 0.41, -0.01, 0.08, 0.3, 0.7, 0.1, 0.2, 0.73, 0.11, -0.22,
+                 0.4, 3, 7, 0.7, 0.4), tolerance=1e-4)
+
+  expect_equal(reorder_B_columns(mod222logistiktb, perm=c(1, 2), calc_std_errors=FALSE)$params,
+               mod222logistiktb$params, tolerance=1e-4)
+
+  expect_equal(reorder_B_columns(mod222logistiktb, perm=c(2, 1), calc_std_errors=FALSE)$model$B_constraints,
+               matrix(c(0, 1, 1, NA), nrow=2), tolerance=1e-4)
 })
 
 
@@ -384,4 +445,22 @@ test_that("swap_B_signs works correctly", {
                matrix(c(-1, NA, 0, -1), nrow=2), tolerance=1e-4)
   expect_equal(swap_B_signs(mod222logistitb, which_to_swap=1, calc_std_errors=FALSE)$model$B_constraints,
                matrix(c(-1, NA, 0, 1), nrow=2), tolerance=1e-4)
+
+  # ind_skewed_t
+  expect_equal(swap_B_signs(mod123exoikt, which_to_swap=1, calc_std_errors=FALSE)$params,
+               c(0.10741, 0.13813, -0.12092, 3.48957, 0.60615, 0.45646, 0.87227, -0.01595, 0.14124, -0.08611, 0.61865, 0.34311,
+                 -0.02047, 0.025, 0.97548, 0.74976, 0.02187, 0.29213, -1.55165, 0.58245, -0.00696, -0.07261, 0.02021, 0.96883,
+                 -0.217, 0.542, -0.891, 0.596, 1.636, 0.689, -1.281, -0.213, 1.897, -1.777, -0.567, -0.016, 0.383, -0.045, 0.034,
+                 0.169, 1.165, -0.044, 7, 3, 13, -0.10000, 0.2, 0.3), tolerance=1e-4)
+
+  expect_equal(swap_B_signs(mod222logistiktb, which_to_swap=2, calc_std_errors=FALSE)$params,
+               c(0.720966, 0.810858, 0.22, 0.06, -0.15, 0.39, 0.41, -0.01, 0.08, 0.3, 0.1, 0.2, -0.7, 0.11, -0.22, -0.73, 0.4, 7, 3,
+                 0.4, 0.7), tolerance=1e-4)
+
+  expect_equal(swap_B_signs(mod222logistiktb, which_to_swap=2, calc_std_errors=FALSE)$model$B_constraints,
+               matrix(c(1, NA, 0, -1), nrow=2), tolerance=1e-4)
+  expect_equal(swap_B_signs(mod222logistiktb, which_to_swap=1:2, calc_std_errors=FALSE)$model$B_constraints,
+               matrix(c(-1, NA, 0, -1), nrow=2), tolerance=1e-4)
+  expect_equal(swap_B_signs(mod222logistiktb, which_to_swap=1, calc_std_errors=FALSE)$model$B_constraints,
+               matrix(c(-1, NA, 0, 1), nrow=2), tolerance=1e-4)
 })
