@@ -324,8 +324,54 @@ B3_232 <- matrix(c(0.9, 0.3, -0.2, 0.4), nrow=2)
 theta_232exoit <- c(phi10_232, phi20_232, phi30_232, vec(A11_232), vec(A12_232), vec(A21_232), vec(A22_232),
                     vec(A31_232), vec(A32_232), vec(B1_232), vec(B2_232), vec(B3_232), dfs_232_1_1)
 
-###############
 
+################
+### ind_skewed_t
+
+# p=1, M=1, d=2, weight_function="threshold", weightfun_pars=c(1, 1), cond_dist="ind_skew_t"
+dfls_112 <- c(3, 7, 0.3, 0.4)
+B1_112 <- matrix(c(0.5, 0.2, -0.7, 0.3), nrow=2)
+theta_112ikt <- c(phi10_112, vec(A11_112), vec(B1_112), dfls_112)
+
+# p=2, M=2, d=2, cond_dist="ind_skewed_t", weight_function="logistic", weightfun_pars=c(2, 1)
+dfls_222_2_1 <- c(3, 7, 0, -0.4)
+B1_222 <- matrix(c(0.5, 0.2, -0.1, 0.3), nrow=2)
+B2_222 <- matrix(c(0.4, -0.1, -0.2, 0.3), nrow=2)
+theta_222logistikt_2_1 <- c(phi10_222, phi20_222, vec(A11_222), vec(A12_222), vec(A21_222), vec(A22_222),
+                            vec(B1_222), vec(B2_222), c_and_gamma_222_2_1, dfls_222_2_1)
+
+# p=1, M=2, d=2, weight_function="mlogit", weightfun_pars=list(vars=1, lags=1), cond_dist="ind_skewed_t"
+dfls_122_1_1 <- c(4, 13, 0.1, -0.2)
+B1_122 <- matrix(c(1.2, -0.3, 0.7, 0.1), nrow=2)
+B2_122 <- matrix(c(0.5, 0.2, -0.1, 3.1), nrow=2)
+theta_122logikt_1_1 <- c(phi10_122, phi20_122, vec(A11_122), vec(A21_122), vec(B1_122), vec(B2_122), gamma1_122_1_1, dfls_122_1_1)
+
+# p=1, M=2, d=3, weight_function="exponential", weightfun_pars=c(1, 1), cond_dist="ind_skewed_t"
+dfls_123_1_1 <- c(10, 12, 3, 0.1, 0, -0.1)
+B1_123 <- matrix(c(1.0, 0.3, 0.1, -0.8, 1.1, -0.5, -0.1, -0.2, 0.4), nrow=3)
+B2_123 <- matrix(c(0.3, -0.2, -0.7, -0.8, 1.2, 0.5, 0.1, -0.2, 1.1), nrow=3)
+theta_123expikt_1_1 <- c(phi10_123, phi20_123, vec(A11_123), vec(A21_123), vec(B1_123),
+                         vec(B2_123), c_and_gamma_123_1_1, dfls_123_1_1)
+
+# p=1, M=3, d=2, weight_function="threshold", weightfun_pars=c(1, 1), cond_dist="ind_skewed_t"
+dfls_132_1_1 <- c(30, 6, 0.1, 0.2)
+B1_132 <- matrix(c(0.6, 0.2, -0.1, 0.7), nrow=2)
+B2_132 <- matrix(c(0.4, -0.1, -0.2, 0.5), nrow=2)
+B3_132 <- matrix(c(0.9, -0.5, 0.2, 0.4), nrow=2)
+theta_132thresikt_1_1 <- c(phi10_132, phi20_132, phi30_132, vec(A11_132), vec(A21_132), vec(A31_132),
+                           vec(B1_132), vec(B2_132), vec(B3_132), r1_132_1_1, r2_132_1_1, dfls_132_1_1)
+
+# p=2, M=3, d=2, weight_function="exogenous", weightfun_pars=matrix(cbind(c(0.4, 0, 0.5), c(0.3, 1, 0.2), c(0.3, 0, 0.3))),
+# cond_dist="ind_skewed_t"
+dfls_232_1_1 <- c(3, 6, -0.1, -0.2)
+B1_232 <- matrix(c(0.6, -0.2, 0.1, 0.7), nrow=2)
+B2_232 <- matrix(c(0.4, 0.1, 0.2, 0.5), nrow=2)
+B3_232 <- matrix(c(0.9, 0.3, -0.2, 0.4), nrow=2)
+theta_232exoikt <- c(phi10_232, phi20_232, phi30_232, vec(A11_232), vec(A12_232), vec(A21_232), vec(A22_232),
+                     vec(A31_232), vec(A32_232), vec(B1_232), vec(B2_232), vec(B3_232), dfls_232_1_1)
+
+
+#####################
 ### Structural models
 # (recursively identified models use the same parametrization as reduced form models)
 
@@ -362,10 +408,10 @@ theta_232threstsh_1_1 <- c(phi10_232, phi20_232, phi30_232, vec(A11_232), vec(A1
 
 ### ind_Student
 
-# p=2, M=2, d=2, cond_dist="ind_Student", weight_function="logistic", weightfun_pars=c(2, 1), identification="non-Gaussianity"
+# p=2, M=2, d=2, cond_dist="ind_skewed_t", weight_function="logistic", weightfun_pars=c(2, 1), identification="non-Gaussianity"
 theta_222logistitng_2_1 <- theta_222logistit_2_1
 
-# p=1, M=2, d=2, weight_function="mlogit", weightfun_pars=list(vars=1, lags=1), cond_dist="ind_Student", identification="non-Gaussianity"
+# p=1, M=2, d=2, weight_function="mlogit", weightfun_pars=list(vars=1, lags=1), cond_dist="ind_skewed_t", identification="non-Gaussianity"
 theta_122logitng_1_1 <- theta_122logit_1_1
 
 # p=1, M=2, d=3, weight_function="exponential", weightfun_pars=c(1, 1), cond_dist="ind_Student", identification="non-Gaussianity"
@@ -378,6 +424,24 @@ theta_132thresitng_1_1 <- theta_132thresit_1_1
 # cond_dist="ind_Student", identification="non-Gaussianity"
 theta_232exoitng <- theta_232exoit
 
+### ind_skewed_t
+
+# p=2, M=2, d=2, cond_dist="ind_skewed_t", weight_function="logistic", weightfun_pars=c(2, 1), identification="non-Gaussianity"
+theta_222logistiktng_2_1 <- theta_222logistikt_2_1
+
+# p=1, M=2, d=2, weight_function="mlogit", weightfun_pars=list(vars=1, lags=1), cond_dist="ind_skewed_t", identification="non-Gaussianity"
+theta_122logiktng_1_1 <- theta_122logikt_1_1
+
+# p=1, M=2, d=3, weight_function="exponential", weightfun_pars=c(1, 1), cond_dist="ind_skewed_t", identification="non-Gaussianity"
+theta_123expiktng_1_1 <- theta_123expikt_1_1
+
+# p=1, M=3, d=2, weight_function="threshold", weightfun_pars=c(1, 1), cond_dist="ind_skewed_t", identification="non-Gaussianity"
+theta_132thresiktng_1_1 <- theta_132thresikt_1_1
+
+# p=2, M=3, d=2, weight_function="exogenous", weightfun_pars=matrix(cbind(c(0.4, 0, 0.5), c(0.3, 1, 0.2), c(0.3, 0, 0.3))),
+# cond_dist="ind_skewed_t", identification="non-Gaussianity"
+theta_232exoiktng <- theta_232exoikt
+
 test_that("pick_phi0 works correctly", {
 
   expect_equal(pick_phi0(M=2, d=3, params=theta_123exo)[,1], phi10_123)
@@ -477,6 +541,21 @@ test_that("pick_phi0 works correctly", {
   expect_equal(pick_phi0(M=3, d=2, params=theta_232exoit)[,3], phi30_232)
   expect_equal(pick_phi0(M=2, d=3, params=theta_123expit_1_1)[,1], phi10_123)
   expect_equal(pick_phi0(M=2, d=3, params=theta_123expit_1_1)[,2], phi20_123)
+
+  # ind_skew_t
+  expect_equal(pick_phi0(M=1, d=2, params=theta_112ikt)[,1], phi10_112)
+  expect_equal(pick_phi0(M=2, d=2, params=theta_122logikt_1_1)[,1], phi10_122)
+  expect_equal(pick_phi0(M=2, d=2, params=theta_122logikt_1_1)[,2], phi20_122)
+  expect_equal(pick_phi0(M=2, d=2, params=theta_222logistikt_2_1)[,1], phi10_222)
+  expect_equal(pick_phi0(M=2, d=2, params=theta_222logistikt_2_1)[,2], phi20_222)
+  expect_equal(pick_phi0(M=3, d=2, params=theta_132thresikt_1_1)[,1], phi10_132)
+  expect_equal(pick_phi0(M=3, d=2, params=theta_132thresikt_1_1)[,2], phi20_132)
+  expect_equal(pick_phi0(M=3, d=2, params=theta_132thresikt_1_1)[,3], phi30_132)
+  expect_equal(pick_phi0(M=3, d=2, params=theta_232exoikt)[,1], phi10_232)
+  expect_equal(pick_phi0(M=3, d=2, params=theta_232exoikt)[,2], phi20_232)
+  expect_equal(pick_phi0(M=3, d=2, params=theta_232exoikt)[,3], phi30_232)
+  expect_equal(pick_phi0(M=2, d=3, params=theta_123expikt_1_1)[,1], phi10_123)
+  expect_equal(pick_phi0(M=2, d=3, params=theta_123expikt_1_1)[,2], phi20_123)
 })
 
 test_that("pick_Ami works correctly", {
@@ -569,6 +648,17 @@ test_that("pick_Ami works correctly", {
   expect_equal(pick_Ami(p=2, M=3, d=2, m=2, i=2, params=theta_232exoit), A22_232)
   expect_equal(pick_Ami(p=2, M=3, d=2, m=3, i=2, params=theta_232exoit), A32_232)
 
+  # ind_skew_t
+  expect_equal(pick_Ami(p=1, M=1, d=2, m=1, i=1, params=theta_112ikt), A11_112)
+  expect_equal(pick_Ami(p=1, M=2, d=3, m=1, i=1, params=theta_123expikt_1_1), A11_123)
+  expect_equal(pick_Ami(p=1, M=2, d=3, m=2, i=1, params=theta_123expikt_1_1), A21_123)
+  expect_equal(pick_Ami(p=2, M=3, d=2, m=1, i=1, params=theta_232exoikt), A11_232)
+  expect_equal(pick_Ami(p=2, M=3, d=2, m=2, i=1, params=theta_232exoikt), A21_232)
+  expect_equal(pick_Ami(p=2, M=3, d=2, m=3, i=1, params=theta_232exoikt), A31_232)
+  expect_equal(pick_Ami(p=2, M=3, d=2, m=1, i=2, params=theta_232exoikt), A12_232)
+  expect_equal(pick_Ami(p=2, M=3, d=2, m=2, i=2, params=theta_232exoikt), A22_232)
+  expect_equal(pick_Ami(p=2, M=3, d=2, m=3, i=2, params=theta_232exoikt), A32_232)
+
   # unvec=FALSE
   expect_equal(pick_Ami(p=1, M=1, d=2, m=1, i=1, params=theta_112relg, unvec=FALSE), vec(A11_112))
   expect_equal(pick_Ami(p=1, M=2, d=2, m=1, i=1, params=theta_122relg, unvec=FALSE), vec(A11_122))
@@ -675,6 +765,23 @@ test_that("pick_Am works correctly", {
   expect_equal(pick_Am(p=2, M=2, d=2, m=2, params=theta_222logistit_2_1)[, , 2], A22_222)
   expect_equal(pick_Am(p=1, M=2, d=3, m=1, params=theta_123expit_1_1)[, , 1], A11_123)
   expect_equal(pick_Am(p=1, M=2, d=3, m=2, params=theta_123expit_1_1)[, , 1], A21_123)
+
+  # ind_skew_t
+  expect_equal(pick_Am(p=1, M=1, d=2, m=1, params=theta_112ikt)[, , 1], A11_112)
+  expect_equal(pick_Am(p=2, M=3, d=2, m=1, params=theta_232exoikt)[, , 1], A11_232)
+  expect_equal(pick_Am(p=2, M=3, d=2, m=2, params=theta_232exoikt)[, , 1], A21_232)
+  expect_equal(pick_Am(p=2, M=3, d=2, m=3, params=theta_232exoikt)[, , 1], A31_232)
+  expect_equal(pick_Am(p=2, M=3, d=2, m=1, params=theta_232exoikt)[, , 2], A12_232)
+  expect_equal(pick_Am(p=2, M=3, d=2, m=2, params=theta_232exoikt)[, , 2], A22_232)
+  expect_equal(pick_Am(p=2, M=3, d=2, m=3, params=theta_232exoikt)[, , 2], A32_232)
+  expect_equal(pick_Am(p=1, M=2, d=2, m=1, params=theta_122logikt_1_1)[, , 1], A11_122)
+  expect_equal(pick_Am(p=1, M=2, d=2, m=2, params=theta_122logikt_1_1)[, , 1], A21_122)
+  expect_equal(pick_Am(p=2, M=2, d=2, m=1, params=theta_222logistikt_2_1)[, , 1], A11_222)
+  expect_equal(pick_Am(p=2, M=2, d=2, m=1, params=theta_222logistikt_2_1)[, , 2], A12_222)
+  expect_equal(pick_Am(p=2, M=2, d=2, m=2, params=theta_222logistikt_2_1)[, , 1], A21_222)
+  expect_equal(pick_Am(p=2, M=2, d=2, m=2, params=theta_222logistikt_2_1)[, , 2], A22_222)
+  expect_equal(pick_Am(p=1, M=2, d=3, m=1, params=theta_123expikt_1_1)[, , 1], A11_123)
+  expect_equal(pick_Am(p=1, M=2, d=3, m=2, params=theta_123expikt_1_1)[, , 1], A21_123)
 })
 
 test_that("pick_allA works correctly", {
@@ -768,6 +875,23 @@ test_that("pick_allA works correctly", {
   expect_equal(pick_allA(p=2, M=2, d=2, params=theta_222logistit_2_1)[, , 2, 2], A22_222)
   expect_equal(pick_allA(p=1, M=2, d=3, params=theta_123expit_1_1)[, , 1, 1], A11_123)
   expect_equal(pick_allA(p=1, M=2, d=3, params=theta_123expit_1_1)[, , 1, 2], A21_123)
+
+  # ind_skew_t
+  expect_equal(pick_allA(p=1, M=1, d=2, params=theta_112ikt)[, , 1, 1], A11_112)
+  expect_equal(pick_allA(p=2, M=3, d=2, params=theta_232exoikt)[, , 1, 2], A11_232)
+  expect_equal(pick_allA(p=2, M=3, d=2, params=theta_232exoikt)[, , 1, 2], A21_232)
+  expect_equal(pick_allA(p=2, M=3, d=2, params=theta_232exoikt)[, , 1, 3], A31_232)
+  expect_equal(pick_allA(p=2, M=3, d=2, params=theta_232exoikt)[, , 2, 2], A12_232)
+  expect_equal(pick_allA(p=2, M=3, d=2, params=theta_232exoikt)[, , 2, 2], A22_232)
+  expect_equal(pick_allA(p=2, M=3, d=2, params=theta_232exoikt)[, , 2, 3], A32_232)
+  expect_equal(pick_allA(p=1, M=2, d=2, params=theta_122logikt_1_1)[, , 1, 1], A11_122)
+  expect_equal(pick_allA(p=1, M=2, d=2, params=theta_122logikt_1_1)[, , 1, 2], A21_122)
+  expect_equal(pick_allA(p=2, M=2, d=2, params=theta_222logistikt_2_1)[, , 1, 1], A11_222)
+  expect_equal(pick_allA(p=2, M=2, d=2, params=theta_222logistikt_2_1)[, , 2, 1], A12_222)
+  expect_equal(pick_allA(p=2, M=2, d=2, params=theta_222logistikt_2_1)[, , 1, 2], A21_222)
+  expect_equal(pick_allA(p=2, M=2, d=2, params=theta_222logistikt_2_1)[, , 2, 2], A22_222)
+  expect_equal(pick_allA(p=1, M=2, d=3, params=theta_123expikt_1_1)[, , 1, 1], A11_123)
+  expect_equal(pick_allA(p=1, M=2, d=3, params=theta_123expikt_1_1)[, , 1, 2], A21_123)
 })
 
 test_that("pick_Omegas works correctly", {
@@ -847,6 +971,21 @@ test_that("pick_Omegas works correctly", {
   expect_equal(pick_Omegas(p=1, M=3, d=2, params=theta_132thresit_1_1, cond_dist="ind_Student")[, , 2], B2_132)
   expect_equal(pick_Omegas(p=1, M=3, d=2, params=theta_132thresit_1_1, cond_dist="ind_Student")[, , 3], B3_132)
 
+  # ind_skew_t
+  expect_equal(pick_Omegas(p=1, M=1, d=2, params=theta_112ikt, cond_dist="ind_skewed_t")[, , 1], B1_112)
+  expect_equal(pick_Omegas(p=2, M=3, d=2, params=theta_232exoikt, cond_dist="ind_skewed_t")[, , 1], B1_232)
+  expect_equal(pick_Omegas(p=2, M=3, d=2, params=theta_232exoikt, cond_dist="ind_skewed_t")[, , 2], B2_232)
+  expect_equal(pick_Omegas(p=2, M=3, d=2, params=theta_232exoikt, cond_dist="ind_skewed_t")[, , 3], B3_232)
+  expect_equal(pick_Omegas(p=1, M=2, d=2, params=theta_122logikt_1_1, cond_dist="ind_skewed_t")[, , 1], B1_122)
+  expect_equal(pick_Omegas(p=1, M=2, d=2, params=theta_122logikt_1_1, cond_dist="ind_skewed_t")[, , 2], B2_122)
+  expect_equal(pick_Omegas(p=2, M=2, d=2, params=theta_222logistikt_2_1, cond_dist="ind_skewed_t")[, , 1], B1_222)
+  expect_equal(pick_Omegas(p=2, M=2, d=2, params=theta_222logistikt_2_1, cond_dist="ind_skewed_t")[, , 2], B2_222)
+  expect_equal(pick_Omegas(p=1, M=2, d=3, params=theta_123expikt_1_1, cond_dist="ind_skewed_t")[, , 1], B1_123)
+  expect_equal(pick_Omegas(p=1, M=2, d=3, params=theta_123expikt_1_1, cond_dist="ind_skewed_t")[, , 2], B2_123)
+  expect_equal(pick_Omegas(p=1, M=3, d=2, params=theta_132thresikt_1_1, cond_dist="ind_skewed_t")[, , 1], B1_132)
+  expect_equal(pick_Omegas(p=1, M=3, d=2, params=theta_132thresikt_1_1, cond_dist="ind_skewed_t")[, , 2], B2_132)
+  expect_equal(pick_Omegas(p=1, M=3, d=2, params=theta_132thresikt_1_1, cond_dist="ind_skewed_t")[, , 3], B3_132)
+
   # Structural
   expect_equal(pick_Omegas(p=2, M=3, d=2, params=theta_232threst_1_1, identification="recursive")[, , 1], Omega1_232)
   expect_equal(pick_Omegas(p=2, M=3, d=2, params=theta_232threst_1_1, identification="recursive")[, , 2], Omega2_232)
@@ -894,6 +1033,21 @@ test_that("pick_Omegas works correctly", {
   expect_equal(pick_Omegas(p=1, M=3, d=2, params=theta_132thresitng_1_1, cond_dist="ind_Student", identification="non-Gaussianity")[, , 1], B1_132)
   expect_equal(pick_Omegas(p=1, M=3, d=2, params=theta_132thresitng_1_1, cond_dist="ind_Student", identification="non-Gaussianity")[, , 2], B2_132)
   expect_equal(pick_Omegas(p=1, M=3, d=2, params=theta_132thresitng_1_1, cond_dist="ind_Student", identification="non-Gaussianity")[, , 3], B3_132)
+
+  # Ind skew t structural
+  expect_equal(pick_Omegas(p=1, M=1, d=2, params=theta_112ikt, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 1], B1_112)
+  expect_equal(pick_Omegas(p=2, M=3, d=2, params=theta_232exoiktng, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 1], B1_232)
+  expect_equal(pick_Omegas(p=2, M=3, d=2, params=theta_232exoiktng, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 2], B2_232)
+  expect_equal(pick_Omegas(p=2, M=3, d=2, params=theta_232exoiktng, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 3], B3_232)
+  expect_equal(pick_Omegas(p=1, M=2, d=2, params=theta_122logiktng_1_1, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 1], B1_122)
+  expect_equal(pick_Omegas(p=1, M=2, d=2, params=theta_122logiktng_1_1, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 2], B2_122)
+  expect_equal(pick_Omegas(p=2, M=2, d=2, params=theta_222logistiktng_2_1, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 1], B1_222)
+  expect_equal(pick_Omegas(p=2, M=2, d=2, params=theta_222logistiktng_2_1, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 2], B2_222)
+  expect_equal(pick_Omegas(p=1, M=2, d=3, params=theta_123expiktng_1_1, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 1], B1_123)
+  expect_equal(pick_Omegas(p=1, M=2, d=3, params=theta_123expiktng_1_1, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 2], B2_123)
+  expect_equal(pick_Omegas(p=1, M=3, d=2, params=theta_132thresiktng_1_1, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 1], B1_132)
+  expect_equal(pick_Omegas(p=1, M=3, d=2, params=theta_132thresiktng_1_1, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 2], B2_132)
+  expect_equal(pick_Omegas(p=1, M=3, d=2, params=theta_132thresiktng_1_1, cond_dist="ind_skewed_t", identification="non-Gaussianity")[, , 3], B3_132)
 })
 
 test_that("pick_weightpars works correctly", {
@@ -908,6 +1062,8 @@ test_that("pick_weightpars works correctly", {
                                weightfun_pars=matrix(cbind(c(0.4, 0, 0.5), c(0.3, 1, 0.2), c(0.3, 0, 0.3)))), numeric(0))
   expect_equal(pick_weightpars(p=2, M=3, d=2, params=theta_232exoit, weight_function="exogenous", cond_dist="ind_Student",
                                weightfun_pars=matrix(cbind(c(0.4, 0, 0.5), c(0.3, 1, 0.2), c(0.3, 0, 0.3)))), numeric(0))
+  expect_equal(pick_weightpars(p=2, M=3, d=2, params=theta_232exoikt, weight_function="exogenous", cond_dist="ind_skewed_t",
+                               weightfun_pars=matrix(cbind(c(0.4, 0, 0.5), c(0.3, 1, 0.2), c(0.3, 0, 0.3)))), numeric(0))
 
   # threshold
   expect_equal(pick_weightpars(p=1, M=2, d=2, params=theta_122thres_1_1, weight_function="threshold", cond_dist="Gaussian",
@@ -926,6 +1082,10 @@ test_that("pick_weightpars works correctly", {
                                weightfun_pars=c(1, 1)), c(r1_132_1_1, r2_132_1_1))
   expect_equal(pick_weightpars(p=1, M=1, d=2, params=theta_122it, weight_function="threshold", cond_dist="ind_Student",
                                weightfun_pars=c(1, 1)), numeric(0))
+  expect_equal(pick_weightpars(p=1, M=3, d=2, params=theta_132thresikt_1_1, weight_function="threshold", cond_dist="ind_skewed_t",
+                               weightfun_pars=c(1, 1)), c(r1_132_1_1, r2_132_1_1))
+  expect_equal(pick_weightpars(p=1, M=1, d=2, params=theta_122ikt, weight_function="threshold", cond_dist="ind_skewed_t",
+                               weightfun_pars=c(1, 1)), numeric(0))
 
   # exponential
   expect_equal(pick_weightpars(p=1, M=2, d=2, params=theta_122exp_1_1, weight_function="exponential", cond_dist="Gaussian",
@@ -942,6 +1102,8 @@ test_that("pick_weightpars works correctly", {
                                weightfun_pars=c(1, 1)), c_and_gamma_122_1_1)
   expect_equal(pick_weightpars(p=1, M=2, d=3, params=theta_123expit_1_1, weight_function="exponential", cond_dist="ind_Student",
                                weightfun_pars=c(1, 1)), c_and_gamma_123_1_1)
+  expect_equal(pick_weightpars(p=1, M=2, d=3, params=theta_123expikt_1_1, weight_function="exponential", cond_dist="ind_skewed_t",
+                               weightfun_pars=c(1, 1)), c_and_gamma_123_1_1)
 
   # logistic
   expect_equal(pick_weightpars(p=1, M=2, d=2, params=theta_122logistic_1_1, weight_function="logistic", cond_dist="Gaussian",
@@ -960,6 +1122,8 @@ test_that("pick_weightpars works correctly", {
                                weightfun_pars=c(2, 1)), c_and_gamma_222_2_1)
   expect_equal(pick_weightpars(p=2, M=2, d=2, params=theta_222logistit_2_1, weight_function="logistic", cond_dist="ind_Student",
                                weightfun_pars=c(2, 1)), c_and_gamma_222_2_1)
+  expect_equal(pick_weightpars(p=2, M=2, d=2, params=theta_222logistikt_2_1, weight_function="logistic", cond_dist="ind_skewed_t",
+                               weightfun_pars=c(2, 1)), c_and_gamma_222_2_1)
 
   # mlogit
   expect_equal(pick_weightpars(p=1, M=2, d=2, params=theta_122log_1_1, weight_function="mlogit", cond_dist="Gaussian",
@@ -988,6 +1152,8 @@ test_that("pick_weightpars works correctly", {
                                weightfun_pars=list(vars=1, lags=1)), gamma1_123_1_1)
   expect_equal(pick_weightpars(p=1, M=2, d=2, params=theta_122logit_1_1, weight_function="mlogit", cond_dist="ind_Student",
                                weightfun_pars=list(vars=1, lags=1)), gamma1_122_1_1)
+  expect_equal(pick_weightpars(p=1, M=2, d=2, params=theta_122logikt_1_1, weight_function="mlogit", cond_dist="ind_skewed_t",
+                               weightfun_pars=list(vars=1, lags=1)), gamma1_122_1_1)
 
   # relative_dens
   expect_equal(pick_weightpars(p=1, M=1, d=2, params=theta_112relg, weight_function="relative_dens", cond_dist="Gaussian"), 1)
@@ -1087,6 +1253,23 @@ test_that("pick_regime works correctly", {
                c(phi20_222, vec(A21_222), vec(A22_222), vec(B2_222)))
   expect_equal(pick_regime(p=1, M=2, d=3, m=1, params=theta_123expit_1_1, cond_dist="ind_Student"), c(phi10_123, vec(A11_123), vec(B1_123)))
   expect_equal(pick_regime(p=1, M=2, d=3, m=2, params=theta_123expit_1_1, cond_dist="ind_Student"), c(phi20_123, vec(A21_123), vec(B2_123)))
+
+  # ind_skewed_t
+  expect_equal(pick_regime(p=1, M=1, d=2, m=1, params=theta_112ikt, cond_dist="ind_skewed_t"), c(phi10_112, vec(A11_112), vec(B1_112)))
+  expect_equal(pick_regime(p=2, M=3, d=2, m=1, params=theta_232exoikt, cond_dist="ind_skewed_t"),
+               c(phi10_232, vec(A11_232), vec(A12_232), vec(B1_232)))
+  expect_equal(pick_regime(p=2, M=3, d=2, m=2, params=theta_232exoikt, cond_dist="ind_skewed_t"),
+               c(phi20_232, vec(A21_232), vec(A22_232), vec(B2_232)))
+  expect_equal(pick_regime(p=2, M=3, d=2, m=3, params=theta_232exoikt, cond_dist="ind_skewed_t"),
+               c(phi30_232, vec(A31_232), vec(A32_232), vec(B3_232)))
+  expect_equal(pick_regime(p=1, M=2, d=2, m=1, params=theta_122logikt_1_1, cond_dist="ind_skewed_t"), c(phi10_122, vec(A11_122), vec(B1_122)))
+  expect_equal(pick_regime(p=1, M=2, d=2, m=2, params=theta_122logikt_1_1, cond_dist="ind_skewed_t"), c(phi20_122, vec(A21_122), vec(B2_122)))
+  expect_equal(pick_regime(p=2, M=2, d=2, m=1, params=theta_222logistikt_2_1, cond_dist="ind_skewed_t"),
+               c(phi10_222, vec(A11_222), vec(A12_222), vec(B1_222)))
+  expect_equal(pick_regime(p=2, M=2, d=2, m=2, params=theta_222logistikt_2_1, cond_dist="ind_skewed_t"),
+               c(phi20_222, vec(A21_222), vec(A22_222), vec(B2_222)))
+  expect_equal(pick_regime(p=1, M=2, d=3, m=1, params=theta_123expikt_1_1, cond_dist="ind_skewed_t"), c(phi10_123, vec(A11_123), vec(B1_123)))
+  expect_equal(pick_regime(p=1, M=2, d=3, m=2, params=theta_123expikt_1_1, cond_dist="ind_skewed_t"), c(phi20_123, vec(A21_123), vec(B2_123)))
 })
 
 
@@ -1118,4 +1301,8 @@ test_that("pick_distpars works correctly", {
   expect_equal(pick_distpars(d=2, params=theta_132thresit_1_1, cond_dist="ind_Student"), dfs_132_1_1)
   expect_equal(pick_distpars(d=2, params=theta_132thresit_1_1, cond_dist="ind_Student"), dfs_132_1_1)
   expect_equal(pick_distpars(d=2, params=theta_222logistit_2_1, cond_dist="ind_Student"), dfs_222_2_1)
+  expect_equal(pick_distpars(d=2, params=theta_112ikt, cond_dist="ind_skewed_t"), dfls_112)
+  expect_equal(pick_distpars(d=2, params=theta_132thresikt_1_1, cond_dist="ind_skewed_t"), dfls_132_1_1)
+  expect_equal(pick_distpars(d=2, params=theta_132thresikt_1_1, cond_dist="ind_skewed_t"), dfls_132_1_1)
+  expect_equal(pick_distpars(d=2, params=theta_222logistikt_2_1, cond_dist="ind_skewed_t"), dfls_222_2_1)
 })
